@@ -1,10 +1,10 @@
 ;;; tiny.el --- Quickly generate linear ranges in Emacs
 
-;; Copyright (C) 2013-2015  Free Software Foundation, Inc.
+;; Copyright (C) 2013-2015, 2017  Free Software Foundation, Inc.
 
 ;; Author: Oleh Krehel <ohwoeowho@gmail.com>
 ;; URL: https://github.com/abo-abo/tiny
-;; Version: 0.1.1
+;; Version: 0.2.1
 ;; Keywords: convenience
 
 ;; This file is part of GNU Emacs.
@@ -26,7 +26,7 @@
 ;;
 ;; To set it up, just bind e.g.:
 ;;
-;;     (global-set-key (kbd "C-;") 'tiny-expand)
+;;     (global-set-key (kbd "C-;") #'tiny-expand)
 ;;
 ;; Usage:
 ;; This extension's main command is `tiny-expand'.
@@ -39,54 +39,68 @@
 ;; You can still do the full thing, but +x2 would save you some
 ;; key strokes.
 ;;
-;; You can test out the following snippets
-;; by positioning the point at the end of the expression
-;; and calling `tiny-expand' (default shortcut is C-;):
+;; You can test out the following snippets by positioning the point at
+;; the end of the expression and calling `tiny-expand':
 ;;
-;; m10
-;; m5 10
-;; m5,10
-;; m5 10*xx
-;; m5 10*xx%x
-;; m5 10*xx|0x%x
-;; m25+x?a%c
-;; m25+x?A%c
-;; m97,122(string x)
-;; m97,122stringxx
-;; m97,120stringxupcasex
-;; m97,120stringxupcasex)x
-;; m\n;; 10|%(+ x x) and %(* x x) and %s
-;; m10*2+3x
-;; m\n;; 10expx
-;; m5\n;; 20expx%014.2f
-;; m7|%(expt 2 x)
-;; m, 7|0x%02x
-;; m10|%0.2f
-;; m1\n14|*** TODO http://emacsrocks.com/e%02d.html
-;; m1\n10|convert img%s.jpg -monochrome -resize 50%% -rotate 180 img%s_mono.pdf
-;; (setq foo-list '(m1 11+x96|?%c))
-;; m1\n10listx+x96|convert img%s.jpg -monochrome -resize 50%% -rotate 180 img%c_mono.pdf
-;; m1\n10listxnthxfoo-list|convert img%s.jpg -monochrome -resize 50%% -rotate 180 img%c_mono.pdf
-;; m\n;; 16list*xxx)*xx%s:%s:%s
-;; m\n8|**** TODO Learning from Data Week %(+ x 2) \nSCHEDULED: <%(date "Oct 7" (* x 7))> DEADLINE: <%(date "Oct 14" (* x 7))>
+;;   m10
+;;   m5 10
+;;   m5,10
+;;   m5 10*xx
+;;   m5 10*xx%x
+;;   m5 10*xx|0x%x
+;;   m25+x?a%c
+;;   m25+x?A%c
+;;   m97,122(string x)
+;;   m97,122stringxx
+;;   m97,120stringxupcasex
+;;   m97,120stringxupcasex)x
+;;   m\n;; 10|%(+ x x) and %(* x x) and %s
+;;   m10*2+3x
+;;   m\n;; 10expx
+;;   m5\n;; 20expx%014.2f
+;;   m7|%(expt 2 x)
+;;   m, 7|0x%02x
+;;   m10|%0.2f
+;;   m1\n14|*** TODO http://emacsrocks.com/e%02d.html
+;;   m1\n10|convert img%s.jpg -monochrome -resize 50%% -rotate 180 img%s_mono.pdf
+;;   (setq foo-list '(m1 11+x96|?%c))
+;;   m1\n10listx+x96|convert img%s.jpg -monochrome -resize 50%% -rotate 180 img%c_mono.pdf
+;;   m1\n10listxnthxfoo-list|convert img%s.jpg -monochrome -resize 50%% -rotate 180 img%c_mono.pdf
+;;   m\n;; 16list*xxx)*xx%s:%s:%s
+;;   m\n8|**** TODO Learning from Data Week %(+ x 2) \nSCHEDULED: <%(date "Oct 7" (* x 7))> DEADLINE: <%(date "Oct 14" (* x 7))>
 ;;
 ;; As you might have guessed, the syntax is as follows:
-;; m[<range start:=0>][<separator:= >]<range end>[Lisp expr]|[format expr]
 ;;
-;; x is the default var in the elisp expression.  It will take one by one
-;; the value of all numbers in the range.
+;;   m[<range start:=0>][<separator:= >]<range end>[Lisp expr]|[format expr]
 ;;
-;; | means that elisp expr has ended and format expr has begun.
-;; It can be omitted if the format expr starts with %.
-;; The keys are the same as for format.
-;; In addition %(sexp) forms are allowed.  The sexp can depend on x.
+;;     x is the default var in the elisp expression.  It will take one by one
+;;     the value of all numbers in the range.
 ;;
-;; Note that multiple % can be used in the format expression.
-;; In that case:
-;; * if the Lisp expression returns a list, the members of this list
-;;   are used in the appropriate place.
-;; * otherwise, it's just the result of the expression repeated as
-;;   many times as necessary.
+;;     | means that elisp expr has ended and format expr has begun.
+;;     It can be omitted if the format expr starts with %.
+;;     The keys are the same as for format.
+;;     In addition %(sexp) forms are allowed.  The sexp can depend on x.
+;;
+;;   Note that multiple % can be used in the format expression.
+;;   In that case:
+;;   - if the Lisp expression returns a list, the members of this list
+;;     are used in the appropriate place.
+;;   - otherwise, it's just the result of the expression repeated as
+;;     many times as necessary.
+;;
+;; Alternatively, if user does not want to type in the "tiny
+;; expressions", they can call the `tiny-helper' command that helps
+;; construct the "tiny expression", and then expands that.
+;;
+;; For example, the below two are equivalent:
+;;
+;;  - Type "m2_9+1*x2"
+;;  - M-x tiny-expand
+;;
+;; OR
+;;
+;;  - M-x tiny-helper
+;;  - 9 RET 2 RET _ RET +1*x2 RET RET (user entry in the interactive prompts)
 
 ;;; Code:
 
@@ -112,69 +126,25 @@ to the bounds of the snippet that they matched.
 At the moment, only `tiny-mapconcat' is supported.
 `tiny-mapconcat2' should be added to expand rectangles."
   (interactive)
-  (let ((str (tiny-mapconcat)))
-    (when str
+  (let ((e (tiny-mapconcat)))
+    (when e
+      (goto-char tiny-beg)
       (delete-region tiny-beg tiny-end)
-      (insert str)
-      (tiny-replace-this-sexp))))
+      (insert (eval e t)))))
 
 (defun tiny-setup-default ()
   "Setup shortcuts."
   (global-set-key (kbd "C-;") 'tiny-expand))
 
-(defalias 'tiny--preceding-sexp
-    (if (fboundp 'elisp--preceding-sexp)
-        'elisp--preceding-sexp
-      'preceding-sexp))
-
-;;;###autoload
-(defun tiny-replace-this-sexp ()
-  "Eval and replace the current sexp.
-On error go up list and try again."
-  (interactive)
-  (if (region-active-p)
-      (let ((s (buffer-substring-no-properties
-                (region-beginning)
-                (region-end))))
-        (delete-region (region-beginning)
-                       (region-end))
-        (insert (format "%s" (eval (read s)))))
-    (catch 'success
-      (while t
-        (ignore-errors
-          (unless (looking-back ")" (line-beginning-position))
-            (error "Bad location"))
-          (let ((sexp (tiny--preceding-sexp)))
-            (if (eq (car sexp) 'lambda)
-                (error "Lambda evaluates to itself")
-              (let ((value (eval sexp)))
-                (kill-sexp -1)
-                (insert (format "%s" value))
-                (throw 'success t)))))
-        ;; if can't replace, go up list
-        (condition-case nil
-            (tiny-up-list)
-          (error
-           (message "reached the highest point, couldn't eval.")
-           (throw 'success nil)))))))
-
-(defun tiny-up-list ()
-  "An `up-list' that can exit from string.
-Must throw an error when can't go up further."
-  (interactive)
-  ;; check if inside string
-  (let ((p (nth 8 (syntax-ppss))))
-    (when (eq (char-after p) ?\")
-      ;; go to beginning for string
-      (goto-char p)))
-  (up-list))
+(defun tiny--strip-\n (str)
+  (replace-regexp-in-string "\\\\n" "\n" str))
 
 (defun tiny-mapconcat ()
   "Format output of `tiny-mapconcat-parse'.
 Defaults are used in place of null values."
   (let ((parsed (tiny-mapconcat-parse)))
     (when parsed
-      (let* ((n0 (or (nth 0 parsed) "0"))
+      (let* ((n0 (string-to-number (or (nth 0 parsed) "0")))
              (n1 (nth 1 parsed))
              (s1 (cond ((null n1)
                         " ")
@@ -182,7 +152,7 @@ Defaults are used in place of null values."
                         "")
                        (t
                         n1)))
-             (n2 (nth 2 parsed))
+             (n2 (read (nth 2 parsed)))
              (expr (or (nth 3 parsed) "x"))
              (lexpr (read expr))
              (n-have (if (and (listp lexpr) (eq (car lexpr) 'list))
@@ -192,26 +162,20 @@ Defaults are used in place of null values."
              (n-have (if (zerop n-have) 1 n-have))
              (tes (tiny-extract-sexps (or (nth 4 parsed) "%s")))
              (fmt (car tes))
-             (n-need (cl-count nil (cdr tes)))
              (idx -1)
-             (format-expression
-              (concat "(mapconcat (lambda(x) (let ((lst %s)) (format %S "
-                      (mapconcat (lambda (x)
-                                   (or x
-                                       (if (>= (1+ idx) n-have)
-                                           "x"
-                                         (format "(nth %d lst)" (incf idx)))))
-                                 (cdr tes)
-                                 " ")
-                      ")))(number-sequence %s %s) \"%s\")")))
-        (unless (>= (read n0) (read n2))
-          (format
-           format-expression
-           expr
-           (replace-regexp-in-string "\\\\n" "\n" fmt)
-           n0
-           n2
-           s1))))))
+             (seq (number-sequence n0 n2 (if (>= n0 n2) -1 1))))
+        `(mapconcat (lambda (x)
+                      (let ((lst ,expr))
+                        (format ,(tiny--strip-\n fmt)
+                                ,@(mapcar (lambda (x)
+                                            (if x
+                                                (read x)
+                                              (if (>= (1+ idx) n-have)
+                                                  'x
+                                                `(nth ,(incf idx) lst))))
+                                          (cdr tes)))))
+                    ',seq
+                    ,(tiny--strip-\n s1))))))
 
 (defconst tiny-format-str
   (let ((flags "[+ #-0]\\{0,1\\}")
@@ -240,7 +204,7 @@ Each element of FORMS corresponds to a `format'-style % form in STR.
                 ((and (eq beg (string-match tiny-format-str str beg))
                       (setq fexp (match-string-no-properties 1 str)))
                  (incf beg (length fexp))
-                 (destructuring-bind (sexp . end)
+                 (destructuring-bind (_sexp . end)
                      (read-from-string str beg)
                    (push
                     (replace-regexp-in-string "(date" "(tiny-date"
@@ -262,22 +226,21 @@ m[START][SEPARATOR]END[EXPR]|[FORMAT]
 * END       - integer (required)
 * EXPR      - Lisp expression: function body with argument x (defaults to x)
   Parens are optional if it's unambiguous:
-  - `(* 2 (+ x 3))'  <-> *2+x3
-  - `(exp x)'        <-> expx
+  - `(* 2 (+ x 3))'   <-> *2+x3
+  - `(exp x)'         <-> expx
   A closing paren may be added to resolve ambiguity:
-  - `(* 2 (+ x 3) x) <-> *2+x3)
+  - `(* 2 (+ x 3) x)' <-> *2+x3)
 * FORMAT    - string, `format'-style (defaults to \"%s\")
   | separator can be omitted if FORMAT starts with %.
 
 Return nil if nothing was matched, otherwise
  (START SEPARATOR END EXPR FORMAT)"
   (let ((case-fold-search nil)
-        n1 s1 n2 expr fmt str
-        n-uses)
+        n1 s1 n2 expr fmt str)
     (when (catch 'done
             (cond
               ;; either start with a number
-              ((looking-back "\\bm\\(-?[0-9]+\\)\\([^\n]*?\\)"
+              ((looking-back "\\bm\\(-?[0-9]+\\)\\(.*?\\)"
                              (line-beginning-position))
                (setq n1 (match-string-no-properties 1)
                      str (match-string-no-properties 2)
@@ -335,7 +298,8 @@ Return nil if nothing was matched, otherwise
   (if (equal str "")
       ""
     (ignore-errors
-      (let ((i 0) (j 1)
+      (let ((i 0)
+            (j 1)
             (len (length str))
             sym s out allow-spc
             (n-paren 0)
@@ -373,7 +337,8 @@ Return nil if nothing was matched, otherwise
                   ((progn (setq sym (intern-soft s))
                           (cond
                             ;; general functionp
-                            ((not (eq t (help-function-arglist sym)))
+                            ((and (not (eq t (help-function-arglist sym)))
+                                  (not (eq sym '\,)))
                              (setq expect-fun nil)
                              (setq allow-spc t)
                              ;; (when (zerop n-paren) (push "(" out))
@@ -421,5 +386,106 @@ Optional SHIFT argument is the integer amount of days to shift."
       (setq time (time-add time (days-to-time shift))))
     (format-time-string formatter time)))
 
+;;;###autoload
+(defun tiny-helper (&optional end-val begin-val sep op fmt)
+  "Helper function for `tiny-expand'.
+
+The arguments to this function construct a “tiny expression”
+\"mBSEO|F\" where
+  E is the end value (END-VAL)     - defaults to 0 internally if nil or \"\",
+                                      or 9 if BEGIN-VAL is nil or \"\" too.
+  B is the begin value (BEGIN-VAL) - defaults to 0 internally if nil or \"\".
+  S is the separator (SEP)         - defaults to \" \" if nil or \"\".
+  O is the elisp operation (OP)    - defaults to \"\" if nil.
+  F is the format (FMT)            - defaults to \"\" if nil.
+
+If `tiny' expansion is possible at point, do it.
+Otherwise activate the helper to generate a valid “tiny
+expression” and expand that.
+
+Usage: Call TINY-HELPER, ↵↵↵↵↵            -> 0 1 2 3 4 5 6 7 8 9
+       Call TINY-HELPER, 9↵2↵_↵+1*x2↵↵    -> 5_7_9_11_13_15_17_19
+       Call TINY-HELPER, 15↵1↵↵-30*2x↵%x↵ -> 1c 1a 18 16 14 12 10 e c a 8 6 4 2 0"
+  (interactive
+   (unless (or
+            ;; Use the helper only if tiny expansion is not
+            ;; possible at point and if the buffer is editable.d
+            (barf-if-buffer-read-only)
+            (tiny-mapconcat))
+     (let ((prompt (propertize "tiny-helper: " 'face 'font-lock-function-name-face)))
+       (list (read-string (concat prompt
+                                  "END value "
+                                  "[Hit RET for default=0; "
+                                  "Auto-set to 9 if both begin and end values are 0]: "))
+             (read-string (concat prompt
+                                  "BEGIN value "
+                                  "[Hit RET for default=0; "
+                                  "Has to be *smaller* than the end value]: "))
+             (read-string (concat prompt
+                                  "Separator "
+                                  "[Hit RET for default=Space; "
+                                  "eg: \\n; No math operators like - or = allowed]: "))
+             (read-string (concat prompt
+                                  "Lisp Operation "
+                                  "[Hit RET for default=\"\" (no Lisp operation); "
+                                  "Parentheses are optional; eg: *xx | (+ x ?A) | *2+3x]: "))
+             (read-string (concat prompt
+                                  "Format "
+                                  "[Hit RET for default=\"\" (%0d); "
+                                  "eg: %x | 0x%x | %c | %s | %(+ x x) | "
+                                  "%014.2f | %03d; Parentheses required here for sexps]: "))))))
+  (barf-if-buffer-read-only)  ;Proceed only if the buffer is editable.
+  ;; Use the helper to derive a "tiny expression" if tiny expansion is
+  ;; not possible at point.
+  (when (null (tiny-mapconcat))
+    (let* ((tiny-key-binding (or (substitute-command-keys "\\[tiny-helper]")
+                                 (substitute-command-keys "\\[tiny-expand]")))
+           (end-val (if (null end-val) "" end-val)) ;Initialize to empty strings for non-interactive use.
+           (begin-val (if (null begin-val) "" begin-val))
+           (sep (if (null sep) "" sep))
+           (op (if (null op) "" op))
+           (fmt (if (null fmt) "" fmt))
+           (end-val-num (string-to-number end-val)) ;Note that (string-to-number "") -> 0
+           (begin-val-num (string-to-number begin-val))
+           tiny-expr)
+      ;; BEGIN-VAL and END-VAL sanity check.
+      (cond
+        ((= end-val-num begin-val-num)
+         (if (zerop end-val-num)
+             ;; If both are zero, set the end value to 9 (arbitrarily chosen).
+             (setq end-val "9")
+           (user-error (format "Begin value (%s) and End value (%s) cannot be the same"
+                               begin-val end-val))))
+        ((< end-val-num begin-val-num)
+         (user-error (format "End value (%s) has to be greater than the begin value (%s)"
+                             begin-val end-val))))
+      ;; SEP cannot be an empty string if BEGIN-VAL is a non-empty string.
+      ;; It is OK to not specify BEGIN-VAL if it is 0.
+      (when (and (not (string= begin-val ""))
+                 (string= sep ""))
+        (setq sep " "))
+      ;; When non-empty, prefix FMT with the | char for reading clarity.
+      (when (not (string= fmt ""))
+        (setq fmt (concat "|" fmt)))
+      (setq tiny-expr (concat "m" begin-val sep end-val op fmt))
+      (message (format "This %s expansion can also be done by typing %s and then %s"
+                       (propertize "tiny"
+                                   'face 'font-lock-function-name-face)
+                       (propertize tiny-expr
+                                   'face 'font-lock-keyword-face)
+                       (if (stringp tiny-key-binding)
+                           (propertize tiny-key-binding
+                                       'face 'font-lock-keyword-face)
+                         (concat
+                          (propertize "M-x tiny-helper"
+                                      'face 'font-lock-keyword-face)
+                          " or "
+                          (propertize "M-x tiny-expand"
+                                      'face 'font-lock-keyword-face)))))
+      (insert tiny-expr)
+      (undo-boundary)))
+  (tiny-expand))
+
+
 (provide 'tiny)
 ;;; tiny.el ends here
